'use strict';

exports.__esModule = true;
exports.countTypesTags = countTypesTags;
exports.propsWithNoScriptRender = propsWithNoScriptRender;
exports.wrapTypesToLazyChild = wrapTypesToLazyChild;
exports.wrapTypesToNoScript = wrapTypesToNoScript;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _server = require('react-dom/server');

var _Lazy = require('../components/Lazy');

var _Lazy2 = _interopRequireDefault(_Lazy);

var _LazyChild = require('../components/LazyChild');

var _LazyChild2 = _interopRequireDefault(_LazyChild);

var _LazyGroup = require('../components/LazyGroup');

var _LazyGroup2 = _interopRequireDefault(_LazyGroup);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function countTypesTags(types, children) {
    var count = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

    if (!children) {
        return count;
    }

    _react2.default.Children.forEach(children, function (child) {
        if (!child || child.type === _Lazy2.default || child.type === _LazyGroup2.default || child.type === _LazyChild2.default) {
            return;
        } else if (types.includes(child.type)) {
            count++;
        } else {
            var props = child.props || {};
            count += countTypesTags(types, props.children);
        }
    });

    return count;
}

function propsWithNoScriptRender(children, ltIE9) {
    var props = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    var noscript = (0, _server.renderToStaticMarkup)(_react2.default.createElement('noscript', null, children));

    props.dangerouslySetInnerHTML = {
        __html: !ltIE9 ? noscript : noscript.replace('<noscript>', '<!--[if IE 9]><!--><noscript><!--<![endif]-->').replace('</noscript>', '<!--[if IE 9]><!--></noscript><!--<![endif]-->')
    };

    return props;
}

function wrapTypesToLazyChild(types, children, wrapper, callback, inViewport) {
    if (!children) {
        return children;
    }

    return _react2.default.Children.map(children, function (child) {
        if (!child || child.type === _Lazy2.default || child.type === _LazyGroup2.default || child.type === _LazyChild2.default) {
            return child;
        } else if (types.includes(child.type)) {
            return _react2.default.createElement(
                _LazyChild2.default,
                { callback: callback, wrapper: wrapper },
                inViewport ? child : null
            );
        }
        var props = child.props || {};
        var laziedChildren = wrapTypesToLazyChild(types, props.children, wrapper, callback, inViewport);

        if (laziedChildren !== props.children) {
            return _react2.default.cloneElement(child, null, laziedChildren);
        }

        return child;
    });
}

function wrapTypesToNoScript(types, children, ltIE9, wrapper) {
    if (!children) {
        return children;
    }

    return _react2.default.Children.map(children, function (child) {
        if (!child || child.type === _Lazy2.default || child.type === _LazyGroup2.default || child.type === _LazyChild2.default) {
            return child;
        } else if (types.includes(child.type)) {
            return _react2.default.createElement(wrapper, propsWithNoScriptRender(child, ltIE9));
        }

        var props = child.props || {};
        var noscriptedChildren = wrapTypesToNoScript(types, props.children, ltIE9, wrapper);

        if (noscriptedChildren !== props.children) {
            return _react2.default.cloneElement(child, null, noscriptedChildren);
        }

        return child;
    });
}